@@ -18,6 +18,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 3.0,
+        "cached_input_cost": 0.30,
+        "cache_write_cost": 3.75,
         "output_cost": 15.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -30,6 +32,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 15.0,
+        "cached_input_cost": 1.50,
+        "cache_write_cost": 18.75,
         "output_cost": 75.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -43,6 +47,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 15.0,
+        "cached_input_cost": 1.50,
+        "cache_write_cost": 18.75,
         "output_cost": 75.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -56,6 +62,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 3.0,
+        "cached_input_cost": 0.30,
+        "cache_write_cost": 3.75,
         "output_cost": 15.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -68,6 +76,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 3.0,
+        "cached_input_cost": 0.30,
+        "cache_write_cost": 3.75,
         "output_cost": 15.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -81,6 +91,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 3.0,
+        "cached_input_cost": 0.30,
+        "cache_write_cost": 3.75,
         "output_cost": 15.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -93,6 +105,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 3.0,
+        "cached_input_cost": 0.30,
+        "cache_write_cost": 3.75,
         "output_cost": 15.0,
         "requests_per_minute": 4_000,
         "tokens_per_minute": 400_000,
@@ -116,8 +130,10 @@ ANTHROPIC_MODELS = {
         "api_key_env_var": "ANTHROPIC_API_KEY",
         "supports_json": False,
         "api_spec": "anthropic",
-        "input_cost": 1.00,
-        "output_cost": 5.00,
+        "input_cost": 0.8,
+        "cached_input_cost": 0.08,
+        "cache_write_cost": 1.00,
+        "output_cost": 4.00,
         "requests_per_minute": 20_000,
         "tokens_per_minute": 4_000_000,  # supposed to be this but they fucked up
     },
@@ -129,6 +145,8 @@ ANTHROPIC_MODELS = {
         "supports_json": False,
         "api_spec": "anthropic",
         "input_cost": 0.25,
+        "cache_write_cost": 0.30,
+        "cached_input_cost": 0.03,
         "output_cost": 1.25,
         "requests_per_minute": 10_000,
         "tokens_per_minute": 4_000_000,  # supposed to be this but they fucked up
