@@ -13,7 +13,6 @@ from rich.progress import (
     TaskID,
     TextColumn,
 )
-from rich.text import Text
 from tqdm.auto import tqdm
 
 SECONDS_TO_PAUSE_AFTER_RATE_LIMIT_ERROR = 5
@@ -24,6 +23,7 @@ class StatusTracker:
     max_requests_per_minute: int
     max_tokens_per_minute: int
     max_concurrent_requests: int
+    client_name: str = "LLMClient"
     num_tasks_started: int = 0
     num_tasks_in_progress: int = 0
     num_tasks_succeeded: int = 0
@@ -33,6 +33,13 @@ class StatusTracker:
     total_requests: int = 0
     retry_queue: asyncio.Queue = field(default_factory=asyncio.Queue)
 
+    # Cumulative usage tracking
+    total_cost: float = 0.0
+    total_input_tokens: int = 0  # non-cached input tokens
+    total_cache_read_tokens: int = 0
+    total_cache_write_tokens: int = 0
+    total_output_tokens: int = 0
+
     # Progress bar configuration
     use_progress_bar: bool = True
     progress_bar_total: int | None = None
@@ -131,6 +138,25 @@ class StatusTracker:
         self.num_tasks_in_progress -= 1
         self.num_tasks_failed += 1
 
+    def track_usage(self, response):
+        """Accumulate usage statistics from a completed request.
+
+        Args:
+            response: APIResponse object containing usage and cost information
+        """
+        if response.cost:
+            self.total_cost += response.cost
+
+        if response.usage:
+            self.total_output_tokens += response.usage.output_tokens
+            self.total_input_tokens += response.usage.input_tokens
+
+            if response.usage.cache_read_tokens:
+                self.total_cache_read_tokens += response.usage.cache_read_tokens
+
+            if response.usage.cache_write_tokens:
+                self.total_cache_write_tokens += response.usage.cache_write_tokens
+
     def log_final_status(self):
         # Close progress bar before printing final status
         self.close_progress_bar()
@@ -144,6 +170,22 @@ class StatusTracker:
                 f"{self.num_rate_limit_errors} rate limit errors received. Consider running at a lower rate."
             )
 
+        # Display cumulative usage stats if available
+        if self.total_cost > 0 or self.total_input_tokens > 0 or self.total_output_tokens > 0:
+            usage_parts = []
+            if self.total_cost > 0:
+                usage_parts.append(f"Cost: ${self.total_cost:.4f}")
+            if self.total_input_tokens > 0 or self.total_output_tokens > 0:
+                usage_parts.append(
+                    f"Tokens: {self.total_input_tokens:,} in / {self.total_output_tokens:,} out"
+                )
+            if self.total_cache_read_tokens > 0:
+                usage_parts.append(f"Cache: {self.total_cache_read_tokens:,} read")
+            if self.total_cache_write_tokens > 0:
+                usage_parts.append(f"{self.total_cache_write_tokens:,} write")
+
+            print(" | ".join(usage_parts))
+
     @property
     def pbar(self) -> tqdm | None:
         """Backward compatibility property to access progress bar."""
@@ -187,14 +229,16 @@ class StatusTracker:
 
     def _init_rich_display(self, total: int):
         """Initialize Rich display components."""
-        self._rich_console = Console()
+        self._rich_console = Console(highlight=False)
+        # Escape square brackets so Rich doesn't interpret them as markup
+        description = f"[bold blue]\\[{self.client_name}][/bold blue] Processing..."
         self._rich_progress = Progress(
             SpinnerColumn(),
-            TextColumn("Processing requests..."),
+            TextColumn("[progress.description]{task.description}"),
             BarColumn(),
             MofNCompleteColumn(),
         )
-        self._rich_task_id = self._rich_progress.add_task("requests", total=total)
+        self._rich_task_id = self._rich_progress.add_task(description, total=total)
         self._rich_stop_event = asyncio.Event()
         self._rich_display_task = asyncio.create_task(self._rich_display_updater())
 
@@ -217,12 +261,36 @@ class StatusTracker:
                     total=self.progress_bar_total,
                 )
 
-                tokens_info = f"TPM Capacity: {self.available_token_capacity / 1000:.1f}k/{self.max_tokens_per_minute / 1000:.1f}k"
-                reqs_info = f"RPM Capacity: {int(self.available_request_capacity)}/{self.max_requests_per_minute}"
-                in_progress = f"In Progress: {int(self.num_tasks_in_progress)}"
-                capacity_text = Text(f"{in_progress} • {tokens_info} • {reqs_info}")
+                tokens_info = f"{self.available_token_capacity / 1000:.1f}k/{self.max_tokens_per_minute / 1000:.1f}k TPM"
+                reqs_info = f"{int(self.available_request_capacity)}/{self.max_requests_per_minute} RPM"
+                in_progress = (
+                    f"   [gold3]In Progress:[/gold3] {int(self.num_tasks_in_progress)} "
+                    + ("requests" if self.num_tasks_in_progress != 1 else "request")
+                )
+                capacity_text = (
+                    f"   [gold3]Capacity:[/gold3] {tokens_info} • {reqs_info}"
+                )
 
-                display = Group(self._rich_progress, capacity_text)
+                # Format usage stats
+                usage_parts = []
+                if self.total_cost > 0:
+                    usage_parts.append(f"${self.total_cost:.4f}")
+                if self.total_input_tokens > 0 or self.total_output_tokens > 0:
+                    input_k = self.total_input_tokens / 1000
+                    output_k = self.total_output_tokens / 1000
+                    usage_parts.append(f"{input_k:.1f}k in • {output_k:.1f}k out")
+                if self.total_cache_read_tokens > 0:
+                    cache_k = self.total_cache_read_tokens / 1000
+                    usage_parts.append(f"{cache_k:.1f}k cached")
+
+                usage_text = ""
+                if usage_parts:
+                    usage_text = f"   [gold3]Usage:[/gold3] {' • '.join(usage_parts)}"
+
+                if usage_text:
+                    display = Group(self._rich_progress, in_progress, capacity_text, usage_text)
+                else:
+                    display = Group(self._rich_progress, in_progress, capacity_text)
                 live.update(display)
 
                 await asyncio.sleep(0.1)
@@ -252,7 +320,7 @@ class StatusTracker:
             return
         while not self._manual_stop_event.is_set():
             print(
-                f"Completed {self.num_tasks_succeeded}/{self.progress_bar_total} requests"
+                f"[{self.client_name}] Completed {self.num_tasks_succeeded}/{self.progress_bar_total} requests"
             )
             await asyncio.sleep(self.progress_print_interval)
 
